@@ -3,10 +3,9 @@
 from trytond.pool import PoolMeta
 
 __all__ = ['PurchaseLine']
-__metaclass__ = PoolMeta
 
 
-class PurchaseLine:
+class PurchaseLine(metaclass=PoolMeta):
     __name__ = 'purchase.line'
 
     @classmethod
@@ -14,12 +13,11 @@ class PurchaseLine:
         super(PurchaseLine, cls).__setup__()
         cls._error_messages.update({
                 'missing_account_asset': ('It misses '
-                    'an "Account Asset" on product "%s"!'),
+                    'an "Account Asset" on product "%s".'),
                 })
 
-    def get_invoice_line(self, invoice_type):
-        invoice_lines = super(PurchaseLine, self).get_invoice_line(
-            invoice_type)
+    def get_invoice_line(self):
+        invoice_lines = super(PurchaseLine, self).get_invoice_line()
         if (self.product
                 and self.product.type == 'assets'
                 and self.product.depreciable):
