@@ -1,14 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of this
 # repository contains the full copyright notices and license terms.
-from trytond.model import fields
+from trytond.model import fields, Unique
 from trytond.pyson import Eval
 from trytond.pool import Pool, PoolMeta
+from trytond.transaction import Transaction
 
 __all__ = ['InvoiceLine']
-__metaclass__ = PoolMeta
 
 
-class InvoiceLine:
+class InvoiceLine(metaclass=PoolMeta):
     __name__ = 'account.invoice.line'
     asset = fields.Many2One('account.asset', 'Asset', domain=[
             ('state', '=', 'running'),
@@ -17,7 +17,7 @@ class InvoiceLine:
         states={
             'invisible': (~Eval('is_assets_depreciable', False)
                 | (Eval('_parent_invoice', {}).get('type',
-                        Eval('invoice_type')) != 'out_invoice')),
+                        Eval('invoice_type')) != 'out')),
             },
         depends=['product', 'is_assets_depreciable'])
     is_assets_depreciable = fields.Function(fields.Boolean(
@@ -27,26 +27,28 @@ class InvoiceLine:
     @classmethod
     def __setup__(cls):
         super(InvoiceLine, cls).__setup__()
+        table = cls.__table__()
         cls._sql_constraints += [
-            ('asset_uniq', 'UNIQUE(asset)',
+            ('asset_uniq', Unique(table, table.asset),
                 'Asset can be used only once on invoice line!'),
             ]
 
-    @fields.depends('product', 'invoice', 'invoice_type')
+    @fields.depends('product', 'invoice', 'invoice_type',
+        '_parent_invoice.invoice_date', '_parent_invoice.accounting_date')
     def on_change_product(self):
-        new_values = super(InvoiceLine, self).on_change_product()
+        super(InvoiceLine, self).on_change_product()
         if self.invoice and self.invoice.type:
             type_ = self.invoice.type
         else:
             type_ = self.invoice_type
 
-        if (self.product and type_ in ('in_invoice', 'in_credit_note')
+        if (self.product and type_ == 'in'
                 and self.product.type == 'assets'
                 and self.product.depreciable):
-            new_values['account'] = self.product.account_asset_used.id
-            new_values['account.rec_name'] = \
-                self.product.account_asset_used.rec_name
-        return new_values
+            date = (self.invoice.accounting_date or self.invoice.invoice_date
+                if self.invoice else None)
+            with Transaction().set_context(date=date):
+                self.account = self.product.account_asset_used
 
     @fields.depends('asset', 'unit')
     def on_change_asset(self):
@@ -57,16 +59,10 @@ class InvoiceLine:
             if self.unit:
                 quantity = Uom.compute_qty(self.asset.unit, quantity,
                     self.unit)
-                return {
-                    'quantity': quantity,
-                    }
+                self.quantity = quantity
             else:
-                return {
-                    'quantity': quantity,
-                    'unit': self.unit.id,
-                    'unit.rec_name': self.unit.rec_name,
-                    }
-        return {}
+                self.quantity = quantity
+                self.unit = self.unit
 
     @fields.depends('product')
     def on_change_with_is_assets_depreciable(self, name=None):
@@ -74,8 +70,8 @@ class InvoiceLine:
             return self.product.type == 'assets' and self.product.depreciable
         return False
 
-    def get_move_line(self):
+    def get_move_lines(self):
         Asset = Pool().get('account.asset')
         if self.asset:
             Asset.close([self.asset], account=self.account)
-        return super(InvoiceLine, self).get_move_line()
+        return super(InvoiceLine, self).get_move_lines()
